@@ -1,6 +1,6 @@
 #!/usr/local/bin/perl
 # $File: //member/autrijus/PAR/script/par.pl $ $Author: autrijus $
-# $Revision: #83 $ $Change: 9518 $ $DateTime: 2003/12/31 14:09:50 $ vim: expandtab shiftwidth=4
+# $Revision: #94 $ $Change: 9608 $ $DateTime: 2004/01/04 20:17:28 $ vim: expandtab shiftwidth=4
 
 package __par_pl;
 
@@ -150,8 +150,13 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
-my ($tmpfile, @tmpfiles);
-END { unlink @tmpfiles if @tmpfiles }
+my ($par_temp, $progname, @tmpfile);
+END { if (@tmpfile) {
+    unlink @tmpfile;
+    rmdir $par_temp;
+    $par_temp =~ s{[^\\/]*$}{};
+    rmdir $par_temp;
+} }
 
 BEGIN {
     Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
@@ -163,28 +168,18 @@ if ($ENV{PAR_ARGV_0}) {
     $0 = $ENV{PAR_ARGV_0};
 }
 
-$ENV{PAR_CLEARTEMP} = 1 unless exists $ENV{PAR_CLEARTEMP};
 my $quiet = !$ENV{PAR_DEBUG};
 
-# fix $0 if invoked from PATH
+# fix $progname if invoked from PATH
 my %Config = (
     path_sep    => ($^O =~ /^MSWin/ ? ';' : ':'),
     _exe        => ($^O =~ /^(?:MSWin|OS2|cygwin)/ ? '.exe' : ''),
     _delim      => ($^O =~ /^MSWin|OS2/ ? '\\' : '/'),
 );
 
-if (-s "$0$Config{_exe}") {
-    $0 = "$0$Config{_exe}";
-}
-elsif (!-s $0) {
-    foreach my $dir (split /\Q$Config{path_sep}\E/, $ENV{PATH}) {
-        $dir =~ s/\Q$Config{_delim}\E$//;
-        (($0 = "$dir$Config{_delim}$0$Config{_exe}"), last)
-            if -s "$dir$Config{_delim}$0$Config{_exe}";
-        (($0 = "$dir$Config{_delim}$0"), last)
-            if -s "$dir$Config{_delim}$0";
-    }
-}
+_par_init_env();
+_set_progname();
+_set_par_temp();
 
 # Magic string checking and extracting bundled modules {{{
 my ($start_pos, $data_pos);
@@ -192,7 +187,7 @@ my ($start_pos, $data_pos);
     local $SIG{__WARN__} = sub {};
 
     # Check file type, get start of data section {{{
-    open _FH, $0 or last;
+    open _FH, $progname or last;
     binmode(_FH);
 
     my $buf;
@@ -230,8 +225,6 @@ my ($start_pos, $data_pos);
                 close $out;
                 chmod 0755, $filename;
             }
-            $PAR::Heavy::DLCache{$filename}++;
-            $PAR::Heavy::DLCache{$basename}   =
             $PAR::Heavy::FullCache{$fullname} = $filename;
             $PAR::Heavy::FullCache{$filename} = $fullname;
         }
@@ -246,7 +239,10 @@ my ($start_pos, $data_pos);
         }
         else {
             $require_list{$fullname} =
-            $PAR::Heavy::ModuleCache{$fullname} = \"$buf";
+            $PAR::Heavy::ModuleCache{$fullname} = {
+                buf => $buf,
+                crc => $crc,
+            };
         }
         read _FH, $buf, 4;
     }
@@ -268,18 +264,18 @@ my ($start_pos, $data_pos);
 
         $INC{$module} = "/loader/$filename/$module";
 
-        if (defined(&IO::File::new)) {
+        if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
             my $fh = IO::File->new_tmpfile or die $!;
             binmode($fh);
-            print $fh $$filename;
+            print $fh $filename->{buf};
             seek($fh, 0, 0);
             return $fh;
         }
         else {
-            my ($out, $name) = _tempfile('.pm');
+            my ($out, $name) = _tempfile('.pm', $filename->{crc});
             if ($out) {
                 binmode($out);
-                print $out $$filename;
+                print $out $filename->{buf};
                 close $out;
             }
             open my $fh, $name or die $!;
@@ -375,7 +371,18 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
 
 # Output mode (-O) handling {{{
 if ($out) {
+    require IO::File;
+    require Archive::Zip;
+
     my $par = shift(@ARGV);
+    my $zip = Archive::Zip->new($par) if defined($par);
+    my %env = do {
+        if ($zip and my $meta = $zip->contents('META.yml')) {
+            $meta =~ s/.*^par:$//ms;
+            $meta =~ s/^\S.*//ms;
+            $meta =~ /^  ([^:]+): (.+)$/mg;
+        }
+    };
 
     # Open input and output files {{{
     local $/ = \4;
@@ -386,8 +393,6 @@ if ($out) {
         die "$par is not a PAR file" unless <PAR> eq "PK\003\004";
     }
 
-    require IO::File;
-    require Archive::Zip;
     my $fh = IO::File->new(
 	$out,
 	IO::File::O_CREAT() | IO::File::O_WRONLY() | IO::File::O_TRUNC(),
@@ -402,6 +407,15 @@ if ($out) {
         require PAR::Filter::PodStrip;
         PAR::Filter::PodStrip->new->apply(\$loader, $0)
     }
+    foreach my $key (sort keys %env) {
+        my $val = $env{$key} or next;
+        $val = eval $val if $val =~ /^['"]/;
+        my $magic = "__ENV_PAR_" . uc($key) . "__";
+        my $set = "PAR_" . uc($key) . "=$val";
+        $loader =~ s{$magic( +)}{
+            $magic . $set . (' ' x (length($1) - length($set)))
+        }eg;
+    }
     $fh->print($loader);
     $/ = undef;
     # }}}
@@ -423,7 +437,7 @@ if ($out) {
              $_ ne $Config::Config{privlibexp});
         } @INC;
 
-        if ($^O eq 'MSWin32') { s{\\}{/}g for @inc }
+        if ($Config{_delim} eq '\\') { s{\\}{/}g for @inc }
 
         my %files;
         /^_<(.+)$/ and $files{$1}++ for keys %::;
@@ -462,7 +476,7 @@ if ($out) {
 
             my $content;
             if (ref($file)) {
-                $content = ${$file};
+                $content = $file->{buf};
             }
             else {
                 open FILE, "$file" or die "Can't open $file: $!";
@@ -487,10 +501,7 @@ if ($out) {
     # }}}
 
     # Now write out the PAR and magic strings {{{
-    if (defined($par)) {
-        my $zip = Archive::Zip->new($par);
-        $zip->writeToFileHandle($fh);
-    }
+    $zip->writeToFileHandle($fh) if $zip;
 
     $fh->print(pack('N', $fh->tell - length($loader)));
     $fh->print("\nPAR.pm\n");
@@ -578,55 +589,93 @@ sub require_modules {
     require PAR::Filter::PodStrip;
 }
 
-# N.B. we set PAR_TMP_DIR and PAR_TEMP in myldr/main.c
-my $tmpdir;
-sub tmpdir {
-    return $tmpdir if defined $tmpdir;
-    my @dirlist = (@ENV{qw(PAR_TEMP PAR_TMP_DIR TMPDIR TEMP TMP)}, qw(C:/temp /tmp /));
-    {
-        if (${"\cTAINT"}) { eval {
-            require Scalar::Util;
-            @dirlist = grep { ! Scalar::Util::tainted $_ } @dirlist;
-        } }
+# The C version of this code appears in myldr/mktmpdir.c
+sub _set_par_temp {
+    if ($ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/) {
+        $par_temp = $1;
+        return;
     }
-    foreach (@dirlist) {
-        next unless defined && -d;
-        $tmpdir = $_;
+
+    foreach my $path (
+        (map $ENV{$_}, qw( TMPDIR TEMP TMP )),
+        qw( "C:\\TEMP /tmp . )
+    ) {
+        next unless $path and -d $path and -w $path;
+        my $username = defined(&Win32::LoginName)
+            ? &Win32::LoginName()
+            : $ENV{USERNAME} || $ENV{USER} || 'SYSTEM';
+
+        my $stmpdir = "$path$Config{_delim}par-$username";
+        mkdir $stmpdir, 0755;
+        if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
+            $stmpdir .= "$Config{_delim}cache-$mtime";
+        }
+        else {
+            $ENV{PAR_CLEAN} = 1;
+            $stmpdir .= "$Config{_delim}temp-$$";
+        }
+
+        $ENV{PAR_TEMP} = $stmpdir;
+        mkdir $stmpdir, 0755;
         last;
     }
-    $tmpdir = '' unless defined $tmpdir;
-    return $tmpdir;
+
+    $par_temp = $1 if $ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/;
 }
 
 sub _tempfile {
     my ($ext, $crc) = @_;
     my ($fh, $filename);
     
-    if (defined $crc and !$ENV{PAR_CLEARTEMP}) {
-        $filename = tmpdir() . "/$crc$ext";
-        return (undef, $filename) if (-r $filename);
+    $filename = "$par_temp/$crc$ext";
 
-        open $fh, '>', $filename or die $!;
-    }
-    elsif (defined &File::Temp::tempfile) {
-        # under Win32, the file is created with O_TEMPORARY,
-        # and will be deleted by the C runtime; having File::Temp
-        # delete it has the only effect of giving an ugly warnings
-        ($fh, $filename) = File::Temp::tempfile(
-            SUFFIX      => $ext,
-            UNLINK      => ($^O ne 'MSWin32'),
-        ) or die $!;
+    if ($ENV{PAR_CLEAN}) {
+        unlink $filename if -e $filename;
+        push @tmpfile, $filename;
     }
     else {
-        my $tmpdir = tmpdir();
-        $tmpfile ||= ($$ . '0000');
-        do { $tmpfile++ } while -e ($filename = "$tmpdir/$tmpfile$ext");
-        push @tmpfiles, $filename;
-        open $fh, ">", $filename or die $!;
+        return (undef, $filename) if (-r $filename);
     }
 
+    open $fh, '>', $filename or die $!;
     binmode($fh);
-    return ($fh, $filename);
+    return($fh, $filename);
+}
+
+sub _set_progname {
+    if ($ENV{PAR_PROGNAME} and $ENV{PAR_PROGNAME} =~ /(.+)/) {
+        $progname = $1;
+    }
+    return if -s ($progname ||= $0);
+
+    if (-s "$progname$Config{_exe}") {
+        $ENV{PAR_PROGNAME} = $progname = "$progname$Config{_exe}";
+        return $progname;
+    }
+
+    foreach my $dir (split /\Q$Config{path_sep}\E/, $ENV{PATH}) {
+        $dir =~ s/\Q$Config{_delim}\E$//;
+        (($progname = "$dir$Config{_delim}$progname$Config{_exe}"), last)
+            if -s "$dir$Config{_delim}$progname$Config{_exe}";
+        (($progname = "$dir$Config{_delim}$progname"), last)
+            if -s "$dir$Config{_delim}$progname";
+    }
+
+    $ENV{PAR_PROGNAME} = $progname;
+}
+
+sub _par_init_env {
+    return if $ENV{PAR_INITIALIZED}++;
+
+    my $par_clean = "__ENV_PAR_CLEAN__               ";
+
+    if ($ENV{PAR_TEMP}) {
+        delete $ENV{PAR_CLEAN};
+    }
+    elsif (!$ENV{PAR_CLEAN}) {
+        my $value = substr($par_clean, 12 + length("CLEAN"));
+        $ENV{PAR_CLEAN} = $1 if $value =~ /^PAR_CLEAN=(\S+)/;
+    }
 }
 
 sub outs { warn("@_\n") unless $quiet }
